#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "fuse.h"
#include "fsactions.h"
#include "fsinterface.h"
#include "fusestructs.h"

unsigned char* tempfs;

// Opens the filesystem file for usage
void OpenFS(fuseArgStruct* fuseArgs, char* programPath)
{
    if (!fuseArgs->filefsname)
        FuseUsageError(programPath);
    fuseArgs->fd = open(fuseArgs->fsname, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
    if (fuseArgs->fd == -1)
    {
        perror("open failed");
        exit(EXIT_FAILURE);
    }
    if (zerosize(fuseArgs->fd))
        fuseArgs->newfs = 1;
    if (fuseArgs->newfs)
    {
        if (lseek(fuseArgs->fd, FSSIZE-1, SEEK_SET) == -1)
        {
            perror("seek failed");
            exit(EXIT_FAILURE);
        }
        if (write(fuseArgs->fd, "\0", 1) == -1)
        {
            perror("write failed");
            exit(EXIT_FAILURE);
        }
    }
    return;
}

// Connecting filesystem to memory
FileSystem* SetupFS(fuseArgStruct* fuseArgs)
{
    FileSystem* fs;
    MapFS(fuseArgs->fd);
    LoadFS(&fs);
    if (fuseArgs->newfs)
        FormatFS(fs);
    return fs;
}

// Map filesystem to memory
void MapFS(int fd)
{
    tempfs = mmap(NULL, FSSIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
    if (tempfs == NULL)
    {
        perror("mmap failed");
        exit(EXIT_FAILURE);
    }
    close(fd);
    return;
}

// Unmap filesystem from memory
void UnmapFS(void)
{
    munmap(tempfs, FSSIZE);
    return;
}

// Sets defaults of a new filesystem
void FormatFS(FileSystem* fs)
{
    SetFileSystemDefaults(fs);
    SetupRootDirectory(fs);
    return;
}

// Redirect global pointer to structured local pointer
void LoadFS(FileSystem** fs)
{
    *fs = (FileSystem*) (tempfs);
    return;
}

// Cleanup after using filesystem
void TearDownFS(void)
{
    UnmapFS();
    return;
}

// Return bool of if the size is 0
int zerosize(int fd)
{
    struct stat stats;
    fstat(fd, &stats);
    if(stats.st_size == 0)
        return 1;
    return 0;
}
